--[[
    Table recycling
    $Revision: 242 $
]]--

--[[
Copyright (c) 2009-2011, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local vmajor, vminor = "LibLordFarlander-TableRecycler-1.0", tonumber( ( "$Revision: 242 $" ):match( "(%d+)" ) ) + 90000

local TableRecycler, oldMinor = LibStub:NewLibrary( vmajor, vminor )

if( not TableRecycler ) then
    return
end--if

if( not TableRecycler.tablelist ) then
    TableRecycler.tablelist = {}
end--if

-- Gets a table, filling it with the parameters inserted into it as [name] = value pairs
function TableRecycler:GetTable( ... )
    local usetable = table.remove( self.tablelist ) or {}

    for i = 1, select( "#", ... ), 2 do
        usetable[select( i, ... )] = select( i + 1, ... )
    end--for
    return usetable
end--TableRecycler:GetTable()

-- Gets a table, filling it with the parameters inserted into it
function TableRecycler:GetTableWithInserts( ... )
    local usetable = table.remove( self.tablelist ) or {}

    for i = 1, select( "#", ... ) do
        table.insert( usetable, select( i, ... ) )
    end--for
    return usetable
end--TableRecycler:GetTable()

-- Returns the table as well as all subtables
function TableRecycler:ReturnTable( tableret )
    if( tableret ) then
        for i, v in pairs( tableret ) do
            if( type( v ) == "table" ) then
                self:ReturnTable( v )
            end--if
        end--for
        table.insert( self.tablelist, table.wipe( tableret ) )
    end--if
end--TableRecycler:ReturnTable( tableret )

-- Wipes the table, returning any subtables
function TableRecycler:ClearTable( tableret )
    for i, v in pairs( tableret ) do
        if( type( v ) == "table" ) then
            self:ReturnTable( v )
        end--if
    end--for
    table.wipe( tableret )
end--TableRecycler:ClearTable( tableret )

function TableRecycler.CopyTable( self, to, from )
    if( self ~= TableRecycler ) then
        -- not called as TableRecycler:CopyTable
        -- backwards compatibility
        from = to
        to = self
        self = TableRecycler
    end--if
    for i, v in pairs( from ) do
        if( type( v ) == "table" ) then
            to[i] = self:GetTable()
            self:CopyTable( to[i], v )
        else
            to[i] = v
        end--if
    end--for
end--TableRecycler:CopyTable( to, from )

function TableRecycler.CompareTables( table1, table2 )
    return TableRecycler.SimpleCompareTables( table1, table2 ) and TableRecycler.SimpleCompareTables( table2, table1 )
end--TableRecycler.CompareTables( table1, table2 )

function TableRecycler.SimpleCompareTables( table1, table2 )
    if( table1 and table2 and ( type( table1 ) == "table" ) and ( type( table2 ) == "table" ) ) then
        for i, v in pairs( table1 ) do
            if( type( v ) == "table" ) then
                if( type( table2[i] ) == "table" ) then
                    if( not TableRecycler.SimpleCompareTables( v, table2[i] ) ) then
                        return false
                    end--if
                else
                    return false
                end--if
            elseif( table2[i] ~= v ) then
                return false
            end--if
        end--for
    else
        return false
    end--if
    return true
end--TableRecycler.SimpleCompareTables( table1, table2 )

function TableRecycler.TableCount( t )
    local val = 0

    for i, _ in pairs( t ) do
        val = val + 1
    end--for
    return val
end--TableRecycler.TableCount( t )
